function out = loglike(xx)

% The function calculates the loglikelihood function
% of the model for the parameter in xx. 
% The model in state-space form is given by
%
% y(t) = C(t) + P(t) y(t-1) + D(t) e(t)       State Eq
%
% z(t) = H y(t) + v(t)                        Measurement Eq 
%
% Dimensions and Definitions : 
%  
%                y(t) is n x 1 state vector  
%                z(t) is nobs x 1 vector of observable
%                e(t) is nshocks x 1 vector of structural shocks
%                v(t) is nobs x 1 vector of measurement errors
%
%                V = E[v(t)v(t)'] is nobs x nobs
%
%                Omega = E[e(t)e(t)'] is nshocks x nshocks
%
%                C(t) is (n-1) x 1
%
%                P(t) is (n-1) x (n-1)
%
%                D(t) is (n-1) x nshocks
%
%                H is nobs x (n-1)
% 
%
% Last Modified: 31-Dec-2010
%                Mariano Kulish

global Zobs % Declares the data global

%%
samplesize = length(Zobs);

%% Var-Cov matrix of the fundamental shocks
% Standard deviations into variances
sig2_r = xx(1)^2;
sig2_a = xx(2)^2;
sig2_e = xx(3)^2;
sig2_z = xx(4)^2;

nshocks = 4; % 

Omega = zeros(nshocks);
Omega(1,1) = sig2_r;
Omega(2,2) = sig2_a;
Omega(3,3) = sig2_e;
Omega(4,4) = sig2_z;

%% Var-Cov matrix of the measurement errors
nobs = 3; % number of observables
V = zeros(nobs); % no measurement error

%% Variables and Shocks. 
% 
% Order of variables
r = 1; yhat = 2; g = 3; a = 4; e = 5; x = 6; ppi = 7; const = 8; 
% Dimension of [y(t) 1]
n = 8;
% Order of Shocks in e(t)
%   1     2      3     4  
%  er(t) ea(t)  ee(t) ez(t) 
er = 1; ea =2; ee = 3; ez = 4;
% Dimension of et(t)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initial Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Monetary policy
rho_r = xx(5);
rho_pi = xx(6);
rho_g = xx(7);
rho_x = xx(8);
% Households and firms 
beta = 0.9975; % xx(9);
psi = xx(9);
omega = 0.1; % xx(11);
% Exogenous processes
rho_a = xx(10);
rho_e = xx(11);
% Steady state constants
g_ss = xx(12); % Steady state quarterly growth rate of output per capita
ppi_ss = xx(13); % Steady state quarterly rate of price inflation
r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.
% Set up Binder Pesaran matrices
A0 = zeros(n,n);
A1 = zeros(n,n);
B0 = zeros(n,n);
D0 = zeros(n,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0(1,r) = 1;
A0(1,a) = -(1-rho_a)*(1-omega);
A0(1,x) = 1;
A0(1,const) = ppi_ss - r_ss;
B0(1,x) = 1;
B0(1,ppi) = 1;
% Phillips Curve
% pi(t) = (1-beta)*ppi_ss + beta*Et pi(t+1) + psi x(t) - e(t))
A0(2,e) = 1;
A0(2,x) = -psi;
A0(2,ppi) = 1;
A0(2,const) = (beta-1)*ppi_ss;
B0(2,ppi) = beta;
% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0(3,r) = 1;
A0(3,g) = -rho_g;
A0(3,x) = -rho_x;
A0(3,ppi) = -rho_pi;
A0(3,const) = -((1-rho_r)*r_ss-(rho_pi)*ppi_ss-rho_g*g_ss);
A1(3,r) = rho_r;
D0(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0(4,yhat) = -1;
A0(4,a) = omega;
A0(4,x) = 1;
% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0(5,yhat) = -1;
A0(5,g) = 1;
A0(5,const) = -g_ss;
A1(5,yhat) = -1;
D0(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0(6,a) = 1;
A1(6,a) = rho_a;
D0(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0(7,e) = 1;
A1(7,e) = rho_e;
D0(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0(8,const) = 1;
A1(8,const) = 1;
% AR(1)s are in y(t) so
Gamma = zeros(nshocks);
% D0 is entered above;
D2 = zeros(n, nshocks);
% Solution of Initial Structure
[Q, G, unique, rc, iter] = smatsbp(A0,A1,B0,D0,D2,Gamma);
% Penalise non-uniqueness
if unique == 0
   out =  1e8;
   return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Final Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Monetary policy
rho_r = xx(14);
rho_pi = xx(15); 
rho_g = xx(16); 
ppi_ss = xx(17); % Steady state quarterly rate of price inflation 
r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.
% Set up Binder Pesaran matrices
A0f = zeros(n,n);
A1f = zeros(n,n);
B0f = zeros(n,n);
D0f = zeros(n,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0f(1,r) = 1;
A0f(1,a) = -(1-rho_a)*(1-omega);
A0f(1,x) = 1;
A0f(1,const) = ppi_ss - r_ss;
B0f(1,x) = 1;
B0f(1,ppi) = 1;
% Phillips Curve
% pi(t) = (1-beta)*ppi_ss + beta*Et pi(t+1) + psi x(t) - ehat(t))
A0f(2,e) = 1;
A0f(2,x) = -psi;
A0f(2,ppi) = 1;
A0f(2,const) = (beta-1)*ppi_ss;
B0f(2,ppi) = beta;
% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0f(3,r) = 1;
A0f(3,g) = -rho_g;
A0f(3,x) = -rho_x;
A0f(3,ppi) = -rho_pi;
A0f(3,const) = -((1-rho_r)*r_ss-(rho_pi)*ppi_ss-rho_g*g_ss);
A1f(3,r) = rho_r;
D0f(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0f(4,yhat) = -1;
A0f(4,a) = omega;
A0f(4,x) = 1;
% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0f(5,yhat) = -1;
A0f(5,g) = 1;
A0f(5,const) = -g_ss;
A1f(5,yhat) = -1;
D0f(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0f(6,a) = 1;
A1f(6,a) = rho_a;
D0f(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0f(7,e) = 1;
A1f(7,e) = rho_e;
D0f(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0f(8,const) = 1;
A1f(8,const) = 1;
Gammaf = zeros(nshocks);
% D0 is entered above;
D2f = zeros(n, nshocks);
% Solution of Final Structure

[Qf, Gf, uniquef, rcf, iterf] = smatsbp(A0f,A1f,B0f,D0f,D2f,Gammaf);

% Penalise non-uniqueness
if uniquef == 0
   out =  1e8;
   return;
end

%% Kulish-Pagan Solution for Transition Period
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ta = 140; % Time of the announcement
T = 20;  % Implementation of the change at T + ta
% Construct sequences of time-varying matrices
A0inv = A0\eye(n);
A = A0\A1;
B = A0\B0;
A0s = repmat(A0,T,1);
A0invs = repmat(A0inv,T,1);
A1s = repmat(A1,T,1);
B0s = repmat(B0,T,1);
As = repmat(A,T,1);
Bs = repmat(B,T,1);
Qt = zeros(n,n,T);
Gt = zeros(n,nshocks,T);
% Compute C_T
Qt(:,:,end) = (eye(n)-Bs(end-n+1:end,:)*Qf)\As(end-n+1:end,:);
Gt(:,:,end) = (eye(n)-Bs(end-n+1:end,:)*Qf)\A0invs(end-n+1:end,:)*D0;
% NOTE: in future write this generally to accomodate variations in A and B.
%       that is to have A(t) and B(t) and Ainv(t)
for t = T-1:-1:1,
    Qt(:,:,t) = (eye(n)-B*Qt(:,:,t+1))\A;
    Gt(:,:,t) = (eye(n)-B*Qt(:,:,t+1))\A0inv*D0;
end
%% TIME-VARYING REDUCED FORM MATRICES in tensors

Ct = zeros(n-1,1,samplesize+1);
Pt = zeros(n-1,n-1,samplesize+1);
Dt = zeros(n-1,nshocks,samplesize+1);

for t = 1:ta-1
    Ct(:,:,t) = Q(1:n-1,end);
    Pt(:,:,t) = Q(1:n-1,1:n-1);
    Dt(:,:,t) = G(1:n-1,:);
end

Ct(:,:,ta:ta+T-1) = Qt(1:n-1,end,:);
Pt(:,:,ta:ta+T-1) = Qt(1:n-1,1:n-1,:);
Dt(:,:,ta:ta+T-1) = Gt(1:n-1,:,:);

for t = T+ta:samplesize+1
    Ct(:,:,t) = Qf(1:n-1,end);
    Pt(:,:,t) = Qf(1:n-1,1:n-1);
    Dt(:,:,t) = Gf(1:n-1,:);
end


%% KALMAN FILTER 
%
% Matrix H of measurement equation
% Note: Zobs(t)' = [r(t) ppi(t) g(t)]
%       and variables in y(t)' = 
%       1     2      3     4      5     6      7     
%      r(t)  yhat(t) g(t)  a(t)   e(t)  x(t)   ppi(t) 

H = zeros(3,7);
H(1,1) = 1;
H(2,7) = 1;
H(3,3) = 1;

DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)';

Sigma = (eye((n-1)^2)-(kron(Pt(:,:,1),Pt(:,:,1))))\DOmega(:); % eye(n-1) because I remove the constant

Sigma = reshape(Sigma,n-1,n-1);

    % Construct tensor to store the l x l matrices P(t+1/t)

    St = zeros(n-1,n-1,samplesize);
    St(:,:,1) = Sigma; % Initial condition

 for t=1:(samplesize-1)
     St(:,:,t+1) = Pt(:,:,t+1)*( St(:,:,t)- St(:,:,t)* H' *inv( H*St(:,:,t)*H' + V ) * H*St(:,:,t))* Pt(:,:,t+1)' + Dt(:,:,t+1)*Omega*Dt(:,:,t+1)';
 end
    
%% % Construct tensor to store the Kalman gain matrices

    Kt = zeros(n-1,nobs,samplesize);

    for t=1:samplesize
        Kt(:,:,t) = (Pt(:,:,t+1) * St(:,:,t) * H') / (H * St(:,:,t) * H' + V);
    end

    % Construct series for the forecasts y(t+1/t) 

    y_hat = zeros(n-1,samplesize+1);
    y_hat(:,1) = ((eye(n-1) - Pt(:,:,1)))\Ct(:,:,1); 

    for t=1:samplesize
        y_hat(:,t+1) = Ct(:,:,t+1) + Pt(:,:,t+1)* y_hat(:,t) + Kt(:,:,t)*( Zobs(:,t) - H * y_hat(:,t));
    end

%% LOG-LIKELIHOOD

    U = zeros(nobs,samplesize);
    out = 0;

    for t = 1:samplesize;
        U(:,t) = Zobs(:,t) - H * y_hat(:,t);
        out = out + (nobs/2)*log(2*pi)+(1/2)*log(det(H*St(:,:,t)*H'+V))+(1/2)*(U(:,t)'*inv(H*St(:,:,t)*H'+V)*U(:,t));
    end




if  abs(imag(out)) > 0
    out = real(out) + 1e8;
end
