% modelbp.m
%
% Puts the model in the form of Binder and Pesaran
%
%  A0 y(t) = A1 y(t-1) + B0 E_t y(t+1) + D0 e(t)
%
% and gives a solution of the form
%  
%  y(t) = Q y(t-1) + G e(t)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Equations: 
%
% x(t) = (r_ss-ppi_ss)-(r(t)-Et pi(t+1))+Et x(t+1)+(1-omega)(1-rho_a)a(t) 
%
% pi(t) = (1-beta)*ppi_ss+beta*Et pi(t+1)+psi x(t)-ehat(t))
%
% r(t) = (1-rho_r)*rss-(rho_pi)*ppi_ss-rho_g*g_ss+rho_r*r(t-1) +
%        + rho_pi*pi(t)+rho_g*g(t)+rho_x*x(t)+er(t)
%
% x(t) = yhat(t)-omega a(t)
%
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
%
% a(t) = rho_a*a(t-1) + e_a(t)
%
% e(t) = rho_e*e(t-1) + e_e(t)

% Order of Variables in z(t)
%   1     2      3     4      5     6      7     8
% r(t)  yhat(t) g(t)  a(t)   e(t)  x(t)   ppi(t) const - 8

r = 1; yhat = 2; g = 3; a = 4; e = 5; x = 6; ppi = 7; const = 8; 
% Dimension of z(t)
nvars = 8;

% Order of Shocks in e(t)
%   1     2      3     4      
%  er(t) ea(t), ee(t), ez(t);
er = 1; ea =2; ee = 3; ez = 4;
% Dimension of et(t)
nshocks = 4;

%% Parameters (True Values)
% Monetary policy
rho_r = 0.7;
rho_pi = 0.3;
rho_g = 0.1;
rho_x = 0.05;
% % Households and firms 
beta = 0.9975;
psi = 0.1;
omega = 0.1;
% % Exogenous processes
rho_a = 0.85;
rho_e = 0.85;
% % Steady state constants
g_ss = 0.005; % Steady state quarterly growth rate of output per capita
ppi_ss = 0.05; % Steady state quarterly rate of price inflation
r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.
% % Standard Errors
sig_r = 0.0017;
sig_a = 0.0100;
sig_e = 0.0018;
sig_z = 0.0040;

% Parameters (MLE) uncomment to run bootstrap
% rho_r = 0.700690896552303;
% rho_pi = 0.299418173427158;
% rho_g = 0.100101204251567;
% rho_x = 0.0383977529225238;
% beta = 0.9975;
% psi = 0.0739469803254417;
% omega = 0.1;
% rho_a = 0.819507547780622;
% rho_e = 0.868403447468961;
% g_ss = 0.00478835965846737; 
% ppi_ss = 0.0501045618829056; 
% r_ss = ppi_ss + g_ss - log(beta);
% sig_r = 0.00166785734539781;
% sig_a = 0.0105703879177596;
% sig_e = 0.00162027587012168;
% sig_z = 3.27209161692311e-05;

%% Set up Binder Pesaran matrices
A0 = zeros(nvars,nvars);
A1 = zeros(nvars,nvars);
B0 = zeros(nvars,nvars);
D0 = zeros(nvars,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0(1,r) = 1;
A0(1,a) = -(1-rho_a)*(1-omega);
A0(1,x) = 1;
A0(1,const) = ppi_ss - r_ss;
B0(1,x) = 1;
B0(1,ppi) = 1;

% Phillips Curve
% pi(t) = (1-beta)*ppi_ss + beta*Et pi(t+1) + psi x(t) - ehat(t))
A0(2,e) = 1;
A0(2,x) = -psi;
A0(2,ppi) = 1;
A0(2,const) = (beta-1)*ppi_ss;
B0(2,ppi) = beta;

% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0(3,r) = 1;
A0(3,g) = -rho_g;
A0(3,x) = -rho_x;
A0(3,ppi) = -rho_pi;
A0(3,const) = -((1-rho_r)*r_ss-(rho_pi)*ppi_ss-rho_g*g_ss);
A1(3,r) = rho_r;
D0(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0(4,yhat) = -1;
A0(4,a) = omega;
A0(4,x) = 1;

% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0(5,yhat) = -1;
A0(5,g) = 1;
A0(5,const) = -g_ss;
A1(5,yhat) = -1;
D0(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0(6,a) = 1;
A1(6,a) = rho_a;
D0(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0(7,e) = 1;
A1(7,e) = rho_e;
D0(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0(8,const) = 1;
A1(8,const) = 1;
% The way I've written ar(1) process implies
Gamma = zeros(nshocks);
% D0 is entered above;
D2 = zeros(nvars, nshocks);
%% Compute Solution
[Q, G, unique, rc, iter] = smatsbp(A0,A1,B0,D0,D2,Gamma);
% Steady state
P = Q(1:end-1,1:end-1);
C = Q(1:end-1,end);
y_ss = (eye(nvars-1)-P)\C;
y_ss(nvars) = 1;

%% Final Structure
% Monetary policy (True Values)
rho_r = 1;
rho_pi = 0.8; % rho_pi = 0.4; 0.8
rho_g = 0.3; % rho_g = 0.2; 0.3
rho_x = 0.05;
% Households and firms 
% beta = 0.9975;
% psi = 0.1;
% omega = 0.1;
% Exogenous processes
% rho_a = 0.85;
% rho_e = 0.85;
% Steady state constants
%  g_ss = 0.005;
% Steady state quarterly growth rate of output per capita
ppi_ss = 0.0125; % 0.0125; % 0.025; Steady state quarterly rate of price inflation (5 per cent)
r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.

%Parameters (MLE) Uncomment to run bootstrap
% rho_r = 1.2964033962239;
% rho_pi = 0.89239953376953; 
% rho_g = 0.348543735152734; 
% ppi_ss = 0.0122469917593464; 
% r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.

A0f = zeros(nvars,nvars);
A1f = zeros(nvars,nvars);
B0f = zeros(nvars,nvars);
D0f = zeros(nvars,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0f(1,r) = 1;
A0f(1,a) = -(1-rho_a)*(1-omega);
A0f(1,x) = 1;
A0f(1,const) = ppi_ss - r_ss;
B0f(1,x) = 1;
B0f(1,ppi) = 1;

% Phillips Curve
% pi(t) = (1-beta)*ppi_ss + beta*Et pi(t+1) + psi x(t) - ehat(t))
A0f(2,e) = 1;
A0f(2,x) = -psi;
A0f(2,ppi) = 1;
A0f(2,const) = (beta-1)*ppi_ss;
B0f(2,ppi) = beta;

% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0f(3,r) = 1;
A0f(3,g) = -rho_g;
A0f(3,x) = -rho_x;
A0f(3,ppi) = -rho_pi;
A0f(3,const) = -((1-rho_r)*r_ss-(rho_pi)*ppi_ss-rho_g*g_ss);
A1f(3,r) = rho_r;
D0f(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0f(4,yhat) = -1;
A0f(4,a) = omega;
A0f(4,x) = 1;

% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0f(5,yhat) = -1;
A0f(5,g) = 1;
A0f(5,const) = -g_ss;
A1f(5,yhat) = -1;
D0f(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0f(6,a) = 1;
A1f(6,a) = rho_a;
D0f(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0f(7,e) = 1;
A1f(7,e) = rho_a;
D0f(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0f(8,const) = 1;
A1f(8,const) = 1;

Gammaf = zeros(nshocks);
% D0 is entered above;
D2f = zeros(nvars, nshocks);
%% Compute Solution
[Qf, Gf, uniquef, rcf, iterf] = smatsbp(A0f,A1f,B0f,D0f,D2f,Gammaf);
Pf = Qf(1:end-1,1:end-1);
Cf = Qf(1:end-1,end);
y_ssf = (eye(nvars-1)-Pf)\Cf;

